import matplotlib.pyplot as plt
import pandas as pd

from mskit.plot_kit.drawing_area import set_thousand_separate


def count_bar(data, sort_index=True,
              ylim=None, xlim=None,
              bar_width=0.6, bar_color=None,
              anno_format=',', anno_rotation=90,
              x_ticks=None, x_tick_rotation=0,
              y_tick_thousand=True,
              xlabel='Missed cleavage', ylabel='Number of peptides',
              title='',
              ax=None):
    if ax is None:
        ax = plt.gca()

    if isinstance(data, list):
        ser = pd.Series(dict(data))
    elif isinstance(data, dict):
        ser = pd.Series(data)
    elif isinstance(data, pd.Series):
        ser = data
    else:
        raise TypeError
    if sort_index:
        ser = ser.sort_index()

    data_keys, data_values = list(zip(*pd.Series(ser).items()))
    if x_ticks is not None:
        bar_xsite = x_ticks
    else:
        bar_xsite = list(range(1, len(data_keys) + 1))

    ax.bar(bar_xsite, data_values, width=bar_width, color=bar_color)
    for site, value in zip(bar_xsite, data_values):
        ax.annotate(format(value, anno_format), (site, value / 2),
                    ha='center', rotation=anno_rotation)

    ax.set_xticks(bar_xsite)
    ax.set_xticklabels(data_keys, rotation=x_tick_rotation)

    if y_tick_thousand:
        set_thousand_separate(ax, 'y')
    ax.set_xlabel(xlabel)
    ax.set_ylabel(ylabel)
    ax.set_title(title)



"""
In class XLer

XLers:
    A dict contains all properties of cross-linkers (XLers)
        Key: The abbr name of XLers with all uppercase letters except some special name, e.g. Sulfo-GMBS

FullToAbbr:
    Search the full name of XLers to their abbr

"""


class XLer:
    XLers = {
        'BS3': {
            'FullName': 'BS3',
            'Site-1': ['K', 'N-term'],
            'Site-2': ['K', 'N-term'],
            'Compound': 'C16H18N2Na2O14S2',
            'Composition': '',
            'MaxDistance': 35.
        },

        'DSS': {
            'FullName': 'DISUCCINIMIDYL SUBERATE',
            'Site-1': ['K', 'N-term'],
            'Site-2': ['K', 'N-term'],
            'Compound': 'C16H20N2O8',
            'Composition': '',
            'MaxDistance': 35.
        },

        'EGS': {
            'FullName': 'EGS',
            'Site-1': ['K', 'N-term'],
            'Site-2': ['K', 'N-term'],
            'Compound': 'C18H20N2O12',
            'Composition': '',
            'MaxDistance': 40.
        },

        'EDC': {
            'FullName': 'EDC',
            'Site-1': ['K', 'N-term'],
            'Site-2': ['D', 'E'],
            'Compound': '',
            'Composition': '',
            'MaxDistance': 25.
        },

        'PDH': {
            'FullName': 'PDH',
            'Site-1': ['D', 'E', 'N-term'],
            'Site-2': ['D', 'E'],
            'Compound': '',
            'Composition': '',
            'MaxDistance': 35.
        },

        'Sulfo-GMBS': {
            'FullName': 'Sulfo-GMBS',
            'Site': {
                'Site-1': ['K'],  # TODO Not sure this site
                'Site-2': ['C'],
            },
            'Compound': 'C12H12N2O9S',
            'Composition': '',
            'MaxDistance': 31.
        },

        'ArGO': {
            'FullName': 'ArGO',
            'Site-1': ['R', 'N-term'],
            'Site-2': ['R'],
            'Compound': '',
            'Composition': '',
            'MaxDistance': 43.
        },

        'KArGO': {
            'FullName': 'KArGO',
            'Site-1': ['K', 'N-term'],
            'Site-2': ['R'],
            'Compound': '',
            'Composition': '',
            'MaxDistance': 41.
        },
    }

    FullToAbbr = {desc['FullName']: abbr for abbr, desc in XLers.items()}

    LinkType = {
        'Amine-Amine': ['DSS', 'BS3', 'EGS'],
        'Amine-Carboxyl': ['EDC'],
        'Carboxyl-Carboxyl': ['PDH'],
        'Amine-Sulfhydryl': ['Sulfo-GMBS'],

    }



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Note: To use the 'upload' functionality of this file, you must:
#   $ pipenv install twine --dev

import io
import os
import sys
from shutil import rmtree

from setuptools import find_packages, setup, Command

from mskit import __info__

NAME = __info__.__package_name__
DESCRIPTION = __info__.__short_description__
URL = __info__.__repository__
EMAIL = __info__.__email__
AUTHOR = __info__.__author__
VERSION = __info__.__version__
LICENSE = __info__.__license__
REQUIRES_PYTHON = '>=3.7.0'

REQUIRED = [
    'numpy', 'pandas', 'scipy',
    'matplotlib', 'seaborn',
    'scikit-learn',
    'requests',
    'prettytable', 'tqdm',
]

EXTRAS = {
    'SpeedUp': ['numba'],
    'Parallel': ['joblib', 'dask', 'distributed'],
    'Image': ['opencv-python'],
    'torch': ['pytorch'],
    'tensorflow': ['tensorflow-gpu'],
}

PACKAGEDATA = {
    'unimod': ['package_data/unimod.xml'],
}

classifiers = [
    # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
    'License :: OSI Approved :: MIT License',

    'Development Status :: 2 - Pre-Alpha',

    'Programming Language :: Python :: 3 :: Only',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
    'Programming Language :: Python :: 3.10',
    'Programming Language :: Python :: Implementation :: CPython',

    'Topic :: Scientific/Engineering :: Bio-Informatics',
    'Topic :: Scientific/Engineering :: Visualization',

    'Topic :: Software Development :: Version Control :: Git'
]

here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

# Load the package's __info__.py module as a dictionary.
about = {}
if not VERSION:
    project_slug = NAME.lower().replace("-", "_").replace(" ", "_")
    with open(os.path.join(here, project_slug, '__info__.py')) as f:
        exec(f.read(), about)
else:
    about['__version__'] = VERSION


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system(
            '{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPI via Twine…')
        os.system('twine upload dist/*')

        self.status('Pushing git tags…')
        os.system('git tag v{0}'.format(about['__version__']))
        os.system('git push --tags')

        sys.exit()


setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(
        exclude=[
            "tests",
            "*.tests",
            "*.tests.*",
            "tests.*"]),

    # entry_points={
    #     'console_scripts': ['mycli=mymodule:cli'],
    # },
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    package_data=PACKAGEDATA,
    license=LICENSE,
    classifiers=classifiers,
    cmdclass={
        'upload': UploadCommand,
    },
)



import os

from mskit.inherited_builtins import NonOverwriteDict


def read_prosit_fragment_result(prosit_result):
    """
    '[Oxidation (M)]'
    '[Carbamidomethyl (C)]'
    """
    with open(os.path.abspath(prosit_result), 'r') as prosit_handle:
        prosit_title = prosit_handle.readline()
        title_dict = dict([(__, _) for _, __ in enumerate(prosit_title.strip('\n').split(','))])
        predicted_fragment_data = dict()

        for each_line in prosit_handle:
            if not each_line or each_line == '\n':
                continue
            split_line = each_line.strip('\n').split(',')
            if split_line[0] == '0':
                continue

            loss_type = split_line[title_dict['FragmentLossType']]
            if loss_type != 'noloss':
                continue
            mod_pep = split_line[title_dict['ModifiedPeptide']]
            charge = split_line[title_dict['PrecursorCharge']]
            intensity = split_line[title_dict['RelativeIntensity']]
            fragment_num = split_line[title_dict['FragmentNumber']]
            fragment_type = split_line[title_dict['FragmentType']]
            fragment_charge = split_line[title_dict['FragmentCharge']]

            prec = charge + mod_pep
            fragment_name = '{}{}+{}'.format(fragment_type, fragment_num, fragment_charge)
            if prec not in predicted_fragment_data:
                predicted_fragment_data[prec] = {fragment_name: float(intensity)}
            else:
                predicted_fragment_data[prec][fragment_name] = float(intensity)
    return predicted_fragment_data


def read_prosit_irt_result(prosit_result):
    with open(os.path.abspath(prosit_result), 'r') as prosit_handle:
        prosit_title = prosit_handle.readline()
        title_dict = dict([(__, _) for _, __ in enumerate(prosit_title.strip('\n').split(','))])
        predicted_irt_data = NonOverwriteDict()

        for each_line in prosit_handle:
            if not each_line or each_line == '\n':
                continue
            split_line = each_line.strip('\n').split(',')
            if split_line[0] == '0':
                continue

            irt = split_line[title_dict['iRT']]
            mod_pep = split_line[title_dict['ModifiedPeptide']]

            int_replaced_pep = mod_pep.strip('_').replace('C[Carbamidomethyl (C)]', 'C').replace('M[Oxidation (M)]', '1')
            if '[' in int_replaced_pep:
                continue
            predicted_irt_data[int_replaced_pep] = float(irt)
    return predicted_irt_data


def spectronaut_to_prosit_testset(test_lib_path, test_set_output):
    with open(test_lib_path, 'r') as handle_lib, open(test_set_output, 'w') as handle_test:
        lib_title = handle_lib.readline()
        title_dict = dict([(__, _) for _, __ in enumerate(lib_title.strip('\n').split('\t'))])
        handle_test.write('modified_sequence,collision_energy,precursor_charge\n')

        temp_prec_list = []
        for each_lib_line in handle_lib:
            split_line = each_lib_line.strip('\n').split('\t')
            stripped_pep = split_line[title_dict['StrippedPeptide']]
            if len(stripped_pep) > 30:
                continue

            mod_pep = split_line[title_dict['ModifiedPeptide']]
            charge = split_line[title_dict['PrecursorCharge']]

            current_prec = charge + mod_pep
            if current_prec in temp_prec_list:
                continue
            mod_pep = mod_pep.replace('[Carbamidomethyl (C)]', '')
            mod_pep = mod_pep.replace('[Oxidation (M)]', '(ox)')
            mod_pep = mod_pep.replace('_', '')
            if '[' in mod_pep:
                continue
            handle_test.write('{},{},{}\n'.format(mod_pep, '30', charge))
            temp_prec_list.append(current_prec)


def pdeep_input_to_prosit(pdeep_input_path, prosit_input):
    with open(pdeep_input_path, 'r') as handle_pdeep_input, open(prosit_input, 'w') as handle_prosit:
        pdeep_input_title = handle_pdeep_input.readline()
        title_dict = dict([(__, _) for _, __ in enumerate(pdeep_input_title.strip('\n').split('\t'))])
        handle_prosit.write('modified_sequence,collision_energy,precursor_charge\n')

        temp_prec_list = []
        for each_predict_input_line in handle_pdeep_input:
            split_line = each_predict_input_line.strip('\n').split('\t')
            stripped_pep = split_line[title_dict['peptide']]
            if len(stripped_pep) > 30:
                continue

            charge = split_line[title_dict['charge']]

            mod_info = split_line[title_dict['modification']]
            current_prec = charge + stripped_pep + mod_info
            if current_prec in temp_prec_list:
                continue

            oxi_mod = []
            if mod_info:
                mod_info = mod_info.split(';')[:-1]
                for each_mod in mod_info:
                    if 'Oxidation[M]' in each_mod:
                        oxi_mod.append(int(each_mod.split(',')[0]))
                if oxi_mod:
                    mod_pep = ''
                    previous_oxi_site = 0
                    for each_oxi in oxi_mod:
                        mod_pep += stripped_pep[previous_oxi_site: each_oxi] + '(ox)'
                        previous_oxi_site = each_oxi
                    mod_pep += stripped_pep[previous_oxi_site:]
                    pep = mod_pep
                else:
                    pep = stripped_pep
            else:
                pep = stripped_pep

            handle_prosit.write('{},{},{}\n'.format(pep, '30', charge))
            temp_prec_list.append(current_prec)



from typing import Union


def replace_last_aa(
        pep: Union[str, tuple, list],
        exchange_group=('K', 'R'),
        spcific_mapping=None,
        avoid_mapping_groups: Union[None, tuple] = (('I', 'L'),),
        keep_same=None,
        random_seed=None,
):
    """
    :param pep:
    :param exchange_group:
    :param spcific_mapping:
    :param avoid_mapping_groups:
    :param keep_same:
    :param random_seed:
    """
    pass


protein_list = rk.read_one_col_file()
reverse_protein_list = [_ + '_DecoyReversed' for _ in protein_list]
print(len(protein_list))

total_protein_fasta_path = join_path()
fasta_parser = mskit.seq_process.FastaFile(total_protein_fasta_path)

protein_in_total_fasta = []
for seq_title, seq in fasta_parser.one_protein_generator():
    seq_ident = rk.fasta_title(seq_title)
    if seq_ident in protein_list:
        protein_in_total_fasta.append(seq_title)

len(protein_in_total_fasta)

with open(os.path.splitext(total_protein_fasta_path)[0] + '-DecoyReversed.fasta', 'w') as f:
    for seq_title, seq in fasta_parser.one_protein_generator():
        seq_ident = rk.fasta_title(seq_title)

        if seq_ident in protein_list:
            f.write(seq_title.replace(seq_ident, seq_ident + '_DecoyReversed') + '\n')
            f.write(seq[::-1] + '\n')

        f.write(seq_title + '\n')
        f.write(seq + '\n')





def color_bar(two_side, show_ticks, show_ticklabels, figure_direct, anno_loc, anno_rotation, anno_offsite, ax):
    ax.set_xlim(0, 1)
    ax.set_ylim(0, 1)

    colors = heatmap_color_bar
    n = len(colors)
    for i, c in enumerate(colors):
        ax.plot((i / n, i / n), (0, 1), color=c)

    for anno_index, anno in enumerate(['-2', '0', '2']):
        ax.annotate(anno, (anno_index / 2, 1),
                    textcoords='offset points', xytext=(0, 0.25),
                    va='bottom', ha='center', fontsize=FontSize_Small, rotation=0)

    ax.annotate('z-score intensity', (0.5, 1),
                textcoords='offset points', xytext=(0., 7.),
                va='bottom', ha='center', fontsize=FontSize_Small, rotation=0)



from matplotlib import pyplot as plt

from mskit.plot_kit.annotations import bar_anno, dyna_bar_anno
from mskit.plot_kit.colors import PreColorDict
from mskit.plot_kit.drawing_area import ax_utls


def comp_bar(data_dict, base_key=None, comp_keys=None,
             filter_func=None,  # 对 input data dict 的每个 value 做一步 filter
             base_x_pos=1,
             bar_width=0.4,
             share_color=None, new_color=None, loss_color=None,
             bar_edge_color='grey', bar_edge_width=0.2,
             anno_rotation=90, anno_fontsize=8, anno_ha='center', anno_va='center',
             label_name=None, label_shift=0.1,
             ylabel='', title='',
             ax=None, save=None):
    if ax is None:
        ax = plt.gca()

    if share_color is None:
        share_color = PreColorDict['Blue'][75]
    if new_color is None:
        new_color = PreColorDict['Red'][60]
    if loss_color is None:
        loss_color = PreColorDict['Grey'][60]

    ax_utls.remove_xy_ticks(ax)
    ax_utls.remove_target_spines(('right', 'top'), ax)
    ax_utls.set_bottom_spine_pos0(ax)

    base_data = set(data_dict[base_key])
    if filter_func:
        base_data = filter_func(base_data)
    base_len = len(base_data)
    ax.bar(base_x_pos, base_len, width=bar_width, bottom=0,
           color=share_color, lw=bar_edge_width, edgecolor=bar_edge_color)
    bar_anno(base_len, base_x_pos, y_bottom=0, thousand_separ=True,
             ha=anno_ha, va=anno_va, rotation=anno_rotation, fontsize=anno_fontsize, ax=ax)

    for key_index, each_comp_key in enumerate(comp_keys):
        bar_x_pos = key_index + base_x_pos + 1
        comp_data = set(data_dict[each_comp_key])
        if filter_func:
            comp_data = filter_func(comp_data)

        new_data_num = len(comp_data - base_data)
        share_data_num = len(comp_data & base_data)
        loss_data_num = len(base_data - comp_data)

        # Share
        ax.bar(bar_x_pos, share_data_num, width=bar_width, bottom=0,
               color=share_color, lw=bar_edge_width, edgecolor=bar_edge_color)
        bar_anno(share_data_num, bar_x_pos, y_bottom=0, thousand_separ=True,
                 ha=anno_ha, va=anno_va, rotation=anno_rotation, fontsize=anno_fontsize, ax=ax)

        # New
        ax.bar(bar_x_pos, new_data_num, width=bar_width, bottom=share_data_num,
               color=new_color, lw=bar_edge_width, edgecolor=bar_edge_color)
        dyna_bar_anno(new_data_num, bar_x_pos, y_bottom=share_data_num, thousand_separ=True,
                      ha=anno_ha, va=anno_va, rotation=anno_rotation, fontsize=anno_fontsize,
                      auto_ha_va=True, ax=ax)

        # Loss
        ax.bar(bar_x_pos, -loss_data_num, width=bar_width, bottom=0,
               color=loss_color, lw=bar_edge_width, edgecolor=bar_edge_color)
        dyna_bar_anno(loss_data_num, bar_x_pos, y_height=-loss_data_num, y_bottom=0, thousand_separ=True,
                      ha=anno_ha, va=anno_va, rotation=anno_rotation, fontsize=anno_fontsize,
                      auto_ha_va=True, ax=ax)

    if label_name:
        for label_index, each_name in enumerate(label_name):
            ax.annotate(each_name, (label_index + 1 + bar_width / 2 + label_shift, ax.get_ylim()[0]),
                        rotation=90, va='bottom', ha='left', fontsize=anno_fontsize)

    # ax.set_ylabel(ylabel)  TODO 这里因为删除了 xy ticks 需要单独加一个 text
    ax.set_title(title)

    if save:
        plt.savefig(save + '.Comp.png')



import numpy as np


def remove_outliers(data):
    data = np.array(data)
    p75 = np.percentile(data, 75)
    p25 = np.percentile(data, 25)
    iqr = p75 - p25
    uif = p75 + iqr * 1.5
    dif = p25 - iqr * 1.5
    data = data[(data >= dif) & (data <= uif)]
    return data



import numpy as np
from scipy.stats import pearsonr


def pcc(data1, data2, keep_pvalue=False):
    p = pearsonr(data1, data2)
    if keep_pvalue:
        return p
    else:
        return p[0]


def sa(data1, data2):
    data1 = np.array(data1)
    data2 = np.array(data2)
    norm_data1 = data1 / np.sqrt(sum(np.square(data1)))
    norm_data2 = data2 / np.sqrt(sum(np.square(data2)))
    s = 1 - 2 * (np.arccos(sum(norm_data1 * norm_data2))) / np.pi
    return s
